//--------------------------------------------------------------------
// Pivot Points - Daily (Shifted).mq4
// Copyright  2009, SwingTree Ltd
// 
// This indicator plots horizontal lines at daily pivot levels R3, R2,
// R1, Pivot, S1, S2 & S3, using a day that is advanced (or retarded)
// from the server time (called Pivot Day).
// 
// The only input is the integer "ShiftHrs", which gives the number of
// hours to advance the pivot day. A negative value delays the day for
// pivot calculation. For example, if your server is on New York time
// (GMT-5) and you want pivot points in London time (GMT), then
// ShiftHrs is set to +5 (the default in the code).
//
// Saturday and Sunday trading is not ignored (after all it was real
// trading). If the pivot day shift results in data appearing on
// Saturday, it was really Friday where it occured, so the indicator
// treats it as part of Friday. If the pivot day shift results in data
// appearing on Sunday, it was really Monday where it occured, so the
// indicator treats it as Monday.
//
// It uses an indicator buffer for each line, so pivot levels of
// previous days remain visible. (This has a negative consequence:
// it is not possible to use the same technique to draw intermediate
// levels (S0.5, S1.5, etc.) because of the limit of 8 buffers.)
//
// Labels are shown for the most recent day's levels.
// 
// This indicator can be applied to any chart period, although daily
// pivots on charts with long periods are meaningless.
//
// 2009.07.02
// There was an error in the calculation of R3 & S3,
// which is now corrected.
//
//--------------------------------------------------------------------

#property copyright "Copyright  2009, SwingTree Ltd"
#define PIVOT_NAME "Daily PP"
#define FONT "Arial"
#define PIVOT_COL Yellow

#property indicator_chart_window
#property indicator_buffers 3

#property indicator_color1 PIVOT_COL
#property indicator_color2 PIVOT_COL
#property indicator_color3 PIVOT_COL


// Input(s)
extern int ShiftHrs = 0;   // Pivot day shift
                           // positive value moves pivot day earlier

// Buffers for levels
double Pivot[];
double Pivot1[];
double Pivot2[];

double PDayHigh, PDayLow;
string ThisSymbol;
datetime BarTime, PivotDayStartTime;
int VisibleBars, DayStartBar, LeftMostBar, RightMostBar;


//--------------------------------------------------------------------
// Initialization
//--------------------------------------------------------------------
int init()
{
   // Attach indicator arrays to buffers

   SetIndexBuffer( 0, Pivot);
   SetIndexStyle( 0, DRAW_LINE, STYLE_DOT, 1, PIVOT_COL);
   SetIndexEmptyValue( 0, EMPTY_VALUE );
      SetIndexBuffer( 1, Pivot1);
   SetIndexStyle( 1, DRAW_LINE, STYLE_DOT, 1, PIVOT_COL);
   SetIndexEmptyValue( 1, EMPTY_VALUE );   
   SetIndexBuffer( 2, Pivot2);
   SetIndexStyle( 2, DRAW_LINE, STYLE_DOT, 1, PIVOT_COL);
   SetIndexEmptyValue( 2, EMPTY_VALUE );









   // Catch bad input
   if( MathAbs( ShiftHrs ) > 23 )
   {
      Alert( "ShiftHrs is too large. Reset to 0 hrs. ");
      ShiftHrs = 0;
   }

   ThisSymbol = Symbol();
   PivotDayStartTime = Time[Bars-1];         // the first bar
   
   return(0);
}

//--------------------------------------------------------------------
//| De-initialization                                                |
//--------------------------------------------------------------------
int deinit()
{
   // Remove texts

   ObjectDelete( PIVOT_NAME );


   return(0);
}

//--------------------------------------------------------------------
//| Main iteration                                                   |
//--------------------------------------------------------------------
int start()
{
   int i, Count;
   double Range;

   i = Bars - IndicatorCounted() - 1;

   while(i >= 0)
   {
      // If the pivot day changes...
      if( PivotDay( Time[i+1], ShiftHrs ) != PivotDay( Time[i], ShiftHrs ) )
      {
         // Determine High & Low for the previous Pivot Day
         Count = iBarShift( NULL, 0, PivotDayStartTime ) - i;           // number of bars in the day
         PDayHigh = Close[ iHighest( NULL, 0, MODE_HIGH, Count, i+1 ) ]; // Pivot Day high
         PDayLow = Close[ iLowest( NULL, 0, MODE_LOW, Count, i+1 ) ];     // Pivot Day low

         // Pivot calculations
         Pivot[i] = ( PDayHigh + PDayLow) / 2;    
         Pivot1[i] = PDayHigh;  
         Pivot2[i] = PDayLow;  


         // Don't draw the transition between levels

         //Pivot[i+1] = EMPTY_VALUE;

         
      // Remember when the Day changed over
         PivotDayStartTime = Time[i];
      }
      else     // no change to pivot levels
      {

         Pivot[i] = Pivot[i+1];
         Pivot1[i] = Pivot1[i+1];
         Pivot2[i] = Pivot2[i+1];

      }

      // Move the labels to sensible places
      // If this is the last bar and (it's a new bar or time scale has changed)...
      if( i == 0 && ( BarTime != Time[i] || VisibleBars != WindowBarsPerChart() ) )
      {
         DayStartBar = iBarShift( ThisSymbol, Period(), PivotDayStartTime );
         LeftMostBar = WindowFirstVisibleBar()-7;
         RightMostBar = 15;
         if( DayStartBar < RightMostBar )          // label too close to the right
         {

            ObjectMove( PIVOT_NAME, 0, Time[RightMostBar], Pivot[i] );
            ObjectMove( PIVOT_NAME, 0, Time[RightMostBar], Pivot1[i] );
            ObjectMove( PIVOT_NAME, 0, Time[RightMostBar], Pivot2[i] );

         }
         else if ( DayStartBar > LeftMostBar )     // label too close to the left
         {

            ObjectMove( PIVOT_NAME, 0, Time[LeftMostBar], Pivot[i] );

         }
         else                                      // move it with the bars
         {

            ObjectMove( PIVOT_NAME, 0, PivotDayStartTime, Pivot[i] );

         }
      }
      
      VisibleBars = WindowBarsPerChart();
      BarTime = Time[i];
      i--;
   }

   return(0);
}


//--------------------------------------------------------------------
// int PivotDay( datetime BarTime, datetime ShiftHrs )
// Returns the day of the week for pivot point calculations.
// datetime BarTime: time stamp of the bar of interest
// datetime Shift:   the pivot time - server time shift
//                   i.e. if the time for pivot calculation is ahead
//                   of server time, the shift is positive.
//--------------------------------------------------------------------
int PivotDay( datetime BarTime, datetime ShiftHrs )
{
   int PDay = TimeDayOfWeek( BarTime + ShiftHrs * 3600 );

      if( PDay == 0 ) PDay = 1;      // Count Sunday as Monday
      if( PDay == 6 ) PDay = 5;      // Count Saturday as Friday

   return( PDay );
}

